#include "stdafx.h"
#include "resource.h"
#include "LuxandFaceSDK.h"

LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
INT_PTR CALLBACK	InputName(HWND, UINT, WPARAM, LPARAM);


const char * TrackerMemoryFile = "tracker.dat";              

enum ProgramState { psRecognize, psRemember };
ProgramState programState;
bool NeedClose;

long long IDs[256]; 
long long faceCount = 0;

RECT faceRects[256];
int MouseX = 0;
int MouseY = 0;

char userName[1024];

wchar_t ** CameraList;
int CameraCount = 0;

FSDK_VideoFormatInfo * VideoFormatList = 0;
int VideoFormatCount = 0;


int OpenCamera(int & cameraHandle, int & width, int & height, int & bitsPerPixel) {
	printf("Getting camera list...\n");

	if (0 == FSDK_GetCameraList(&CameraList, &CameraCount))
		for (int i = 0; i < CameraCount; i++)
			wprintf(L"camera: %s\n", CameraList[i]);

	if (0 == CameraCount) {
		MessageBox(0, L"Please attach a camera", L"Error", MB_ICONERROR | MB_OK);
		return -1;
	}

	int CameraIdx = 0; 
	FSDK_GetVideoFormatList(CameraList[CameraIdx], &VideoFormatList, &VideoFormatCount);

	for (int i = 0; i < VideoFormatCount; i++)
		printf("format %d: %dx%d, %d bpp\n", i, VideoFormatList[i].Width, VideoFormatList[i].Height, VideoFormatList[i].BPP);

	int VideoFormat = 0; 
	width = 720 + VideoFormatList[VideoFormat].Width; 
	height = VideoFormatList[VideoFormat].Height;
	bitsPerPixel = VideoFormatList[VideoFormat].BPP; 
	FSDK_SetVideoFormat(CameraList[0], VideoFormatList[VideoFormat]);

	printf("Trying to open the first camera...\n");
	cameraHandle = 0;
	if (FSDKE_OK != FSDK_OpenVideoCamera(CameraList[0], &cameraHandle)) {
		MessageBox(0, L"Error opening the first camera", L"Error", MB_ICONERROR | MB_OK);
		return -2;
	}
	return 0;
}

//INITALIZING THE WINDOW
int InitializeWindow(int width, int height, HDC & dc, HWND & hwnd, HWND & TextBox) { 
	printf("%d  width", width);
	printf("%d  height", height);
	LOGBRUSH brush; 
	brush.lbColor = RGB(200, 200, 200); 
	brush.lbStyle = BS_SOLID;    
	HBRUSH bBrush;
	bBrush = CreateBrushIndirect(&brush);
	WNDCLASSEX wcex; 
	wcex.cbSize = sizeof(WNDCLASSEX);
	wcex.style = 0;
	wcex.lpfnWndProc = WndProc;
	wcex.cbClsExtra = 0;
	wcex.cbWndExtra = 0;
	wcex.hInstance = 0;
	wcex.hIcon = 0;
	wcex.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground = bBrush;
	wcex.lpszMenuName = 0;
	wcex.lpszClassName = L"My Window Class";
	wcex.hIconSm = 0;
	RegisterClassEx(&wcex);



	//CONTROLING MORE WINDOW OPTIONS
	hwnd = CreateWindowEx(WS_EX_TOOLWINDOW, L"My Window Class", L"Autonomous medical systems    -    Live Checkup", WS_SYSMENU, 0, 0, 0, 0, 0, 0, 0, 0);   
	dc = GetDC(hwnd);
	SetWindowPos(hwnd, 0, 0, 0, (width - 10), (height + 210), SWP_NOZORDER);  
	ShowWindow(hwnd, SW_SHOW);
	HPEN hNPen = CreatePen(PS_SOLID, 1, RGB(0, 0, 0));  
	HPEN hOPen = (HPEN)SelectObject(dc, hNPen);
	HBRUSH hOldBrush;
	HBRUSH hNewBrush;
	hNewBrush = (HBRUSH)GetStockObject(NULL_BRUSH);
	hOldBrush = (HBRUSH)SelectObject(dc, hNewBrush);

	UpdateWindow(hwnd);
	TextBox = CreateWindow(L"Static", L"Click face to name it", SS_CENTER | WS_CHILD, 0, 0, 0, 0, hwnd, 0, 0, 0);  
	HFONT MyFont = CreateFont(25, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, L"Microsoft Sans Serif");
	SendMessage(TextBox, WM_SETFONT, WPARAM(MyFont), TRUE);
	SetWindowPos(TextBox, 0, 3 + width / 2 - width, height + 10, 2 * width, 40, SWP_NOZORDER);   
	ShowWindow(TextBox, SW_SHOW);
	UpdateWindow(hwnd);
	return 0;
}



//CURSER BEHAVIOUR:  OUTSIDE & INSIDE THE FACE-RECOGNITION SQUARE
int GetSelectedRectangle() {
	int RectSelected = -1;
	//IF OUTSIDE THE FACE - RECOGNITION SQUARE
	for (int i = 0; i < faceCount; i++)
		if (MouseX >= faceRects[i].left && MouseX <= faceRects[i].right &&
			MouseY >= faceRects[i].top && MouseY <= faceRects[i].bottom)
		{
			RectSelected = i;
			break;
		}

	//IF INSIDE THE FACE-RECOGNITION SQUARE 
	if (RectSelected >= 0) {
		HCURSOR Cur = LoadCursor(NULL, IDC_HAND);
		SetCursor(Cur);
	}
	return RectSelected;
}


void DisplayImage(HDC dc, HImage imageHandle) {
	int width = 0;
	int height = 0;
	FSDK_GetImageWidth(imageHandle, &width);
	FSDK_GetImageHeight(imageHandle, &height);
	HBITMAP hbitmapHandle; 
	FSDK_SaveImageToHBitmap(imageHandle, &hbitmapHandle);
	DrawState(dc, NULL, NULL, (LPARAM)hbitmapHandle, NULL, 0, 16, width, height, DST_BITMAP | DSS_NORMAL);  
	DeleteObject(hbitmapHandle); 
}


void DisplayText(HDC dc, char * name, RECT r) {
	SetBkMode(dc, TRANSPARENT);
	SetTextColor(dc, RGB(0, 102, 204));  //COLOR OF TEXT
	HFONT MyFont = CreateFont(25, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, L"Microsoft Sans Serif");  //TEXT FONT
	SelectObject(dc, MyFont);
	DrawTextA(dc, name, -1, &r, DT_CENTER);
	DeleteObject(MyFont);
}


void DisplayText2(HDC dc, char * name, RECT r) {
	SetBkMode(dc, TRANSPARENT);
	SetTextColor(dc, RGB(255, 255, 255));
	HFONT MyFont = CreateFont(25, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, L"Microsoft Sans Serif");
	SelectObject(dc, MyFont);
	DrawTextA(dc, name, -1, &r, DT_CENTER);
	DeleteObject(MyFont);
}

int _tmain(int argc, _TCHAR* argv[])
{
	if (FSDKE_OK != FSDK_ActivateLibrary("gIPxVS1dAmD4a5SInmaUv8KiSZ7NRUad3KWWU5erxLadTS19WGt6+Vpbwo8R6OEJUYCAe07S6qIoL1C6sAPCE6t+sbCfbDpl3EAVDyHszQofxj7PfRJGnGjTIu7EmMQLI5DxZ032rRy8Qk/OpN91YNnf81Gz1cuvp1LTLgcOcXQ=")) {
		MessageBox(0, L"Please run the License Key Wizard (Start - Luxand - FaceSDK - License Key Wizard)\n", L"Error activating FaceSDK", MB_ICONERROR | MB_OK);
		exit(-1);
	}

	FSDK_Initialize("");
	FSDK_InitializeCapturing();

	int cameraHandle = 0;                    
	int width, height, bitsPerPixel;
	int res = OpenCamera(cameraHandle, width, height, bitsPerPixel);
	if (res < 0) return res;

	// creating a Tracker
	HTracker tracker = 0;
	if (0 != FSDK_LoadTrackerMemoryFromFile(&tracker, TrackerMemoryFile)) 
		FSDK_CreateTracker(&tracker);

	int err = 0; 

	FSDK_SetTrackerMultipleParameters(tracker, "HandleArbitraryRotations=false; DetermineFaceRotationAngle=false; InternalResizeWidth=100; FaceDetectionThreshold=0;      RecognizeFaces = true; DetectExpression = true;       DetectFacialFeatures=true;         DetectGender=true;       ", &err);

	HDC dc;
	HWND hwnd, TextBox;
	InitializeWindow(width, height, dc, hwnd, TextBox); 

	MSG msg = { 0 };
	NeedClose = FALSE;
	programState = psRecognize;

	HPEN FaceRectanglePen = CreatePen(PS_SOLID, 1, RGB(0, 102, 204));
	HBRUSH FaceRectangleBrush = (HBRUSH)GetStockObject(NULL_BRUSH);

	HPEN FeatureCirclesPen = CreatePen(PS_SOLID, 1, RGB(0, 102, 204));
	LOGBRUSH brush;
	brush.lbColor = RGB(0, 102, 204);
	brush.lbStyle = BS_SOLID;
	HBRUSH FeatureCirclesBrush = CreateBrushIndirect(&brush);

	FSDK_SetFaceDetectionParameters(true, true, 256);
	FSDK_SetFaceDetectionThreshold(3);

	RECT ClientRect2;
	GetClientRect(hwnd, &ClientRect2);

	HPEN FaceRectanglePen2 = CreatePen(PS_SOLID, 1, RGB(0, 255, 0));
	HBRUSH FaceRectangleBrush2 = (HBRUSH)GetStockObject(NULL_BRUSH);

	HPEN FeatureCirclesPen2 = CreatePen(PS_SOLID, 1, RGB(0, 0, 255));
	LOGBRUSH brush2;
	brush.lbColor = RGB(0, 0, 255);
	brush.lbStyle = BS_SOLID;
	HBRUSH FeatureCirclesBrush2 = CreateBrushIndirect(&brush);

	SendMessage(hwnd, LB_ADDSTRING, 0, (LPARAM)(L"Press Enter to open image, press Esc to exit ..."));

	TFacePosition facePosition2;
	FSDK_Features facialFeatures2;

	bool ImageOpened2 = false;
	int FaceDetected2 = -1;

	int width2, height2;

	HImage ResizedImageHandle2;
	FSDK_CreateEmptyImage(&ResizedImageHandle2);

	char constName[1024] = { 0 };
	int counterWithFace = 0;
	int counterNoFace = 0;
	int counterForStaticPhote = 0;
	int counterForLogo = 0;
	int totalCounter = 0;
	int temp;


	while (msg.message != WM_QUIT) {
		totalCounter++;
		HImage imageHandle;
		FSDK_GrabFrame(cameraHandle, &imageHandle); 

		FSDK_FeedFrame(tracker, 0, imageHandle, &faceCount, IDs, sizeof(IDs));


		DisplayImage(dc, imageHandle); 

		counterNoFace++;
		if (counterNoFace > 100) {
			counterNoFace = 0;
			temp = FSDK_LoadImageFromFile(&imageHandle, "C:\\0\\000.jpg");

			if (temp != FSDKE_OK) {
				fprintf(stderr, "Error loading file!\n");
				exit(2);
			}

			FaceDetected2 = FSDK_DetectFace(imageHandle, &facePosition2);
			if (FaceDetected2 == FSDKE_OK)
				FSDK_DetectFacialFeaturesInRegion(imageHandle, &facePosition2, &facialFeatures2);

			FSDK_GetImageWidth(imageHandle, &width2);
			FSDK_GetImageHeight(imageHandle, &height2);
			double resizeCoefficient = _min(ClientRect2.right / (double)width2, (ClientRect2.bottom - 16) / (double)height2);

			FSDK_FreeImage(ResizedImageHandle2);
			FSDK_CreateEmptyImage(&ResizedImageHandle2);
			FSDK_ResizeImage(imageHandle, resizeCoefficient, ResizedImageHandle2);

			FSDK_GetImageWidth(ResizedImageHandle2, &width2);
			FSDK_GetImageHeight(ResizedImageHandle2, &height2);

			FSDK_FreeImage(imageHandle);

			if (FaceDetected2 == FSDKE_OK)
			{
				facePosition2.xc = (int)(resizeCoefficient * facePosition2.xc);
				facePosition2.yc = (int)(resizeCoefficient * facePosition2.yc);
				facePosition2.w = (int)(resizeCoefficient * facePosition2.w);

				for (int i = 0; i < FSDK_FACIAL_FEATURE_COUNT; i++)
				{
					facialFeatures2[i].x = (int)(resizeCoefficient * facialFeatures2[i].x);
					facialFeatures2[i].y = (int)(resizeCoefficient * facialFeatures2[i].y);
				}

			}

			ImageOpened2 = true;
			InvalidateRect(hwnd, NULL, TRUE);
		}


		for (int i = 0; i < faceCount; i++) {

			TFacePosition facePosition;
			FSDK_GetTrackerFacePosition(tracker, 0, IDs[i], &facePosition);

			faceRects[i].left = facePosition.xc - (int)(facePosition.w*0.6);
			faceRects[i].top = _min(height, facePosition.yc - (int)(facePosition.w*0.5));
			faceRects[i].right = facePosition.xc + (int)(facePosition.w*0.6);
			faceRects[i].bottom = _min(height, facePosition.yc + (int)(facePosition.w*0.7));
			Rectangle(dc, faceRects[i].left, faceRects[i].top, faceRects[i].right, faceRects[i].bottom);

			char name[1024];
			int res = FSDK_GetAllNames(tracker, IDs[i], name, sizeof(name));


			if (0 == res && strlen(name) > 0) { 
				RECT NameRect;
				NameRect.left = faceRects[i].left - width;
				NameRect.right = faceRects[i].right + width;
				NameRect.top = _min(height, faceRects[i].bottom);
				NameRect.bottom = _min(height, faceRects[i].bottom + 30);
				DisplayText(dc, name, NameRect);
			}

			if ((strcmp(name, "")) && (counterWithFace>40)) {
				strcpy(constName, name);
			}

			printf("%s  =constName", constName);
			counterWithFace++;

			if (counterWithFace>50) {
				counterWithFace = 0;
				counterNoFace = 0;

				HImage imageHandle;

				if (!strcmp(constName, "")) {
					temp = FSDK_LoadImageFromFile(&imageHandle, "C:\\0\\000.jpg");
				}

				if (!strcmp(constName, "301314860")) {
					temp = FSDK_LoadImageFromFile(&imageHandle, "C:\\0\\123.jpg");
				}
				if (!strcmp(constName, "123")) {
					temp = FSDK_LoadImageFromFile(&imageHandle, "C:\\0\\345.jpg");
				}

				if (temp != FSDKE_OK) {
					fprintf(stderr, "Error loading file!\n");
					exit(2);
				}


				FaceDetected2 = FSDK_DetectFace(imageHandle, &facePosition2);
				if (FaceDetected2 == FSDKE_OK)
					FSDK_DetectFacialFeaturesInRegion(imageHandle, &facePosition2, &facialFeatures2);

				FSDK_GetImageWidth(imageHandle, &width2);
				FSDK_GetImageHeight(imageHandle, &height2);
				double resizeCoefficient = _min(ClientRect2.right / (double)width2, (ClientRect2.bottom - 16) / (double)height2);

				FSDK_FreeImage(ResizedImageHandle2);
				FSDK_CreateEmptyImage(&ResizedImageHandle2);
				FSDK_ResizeImage(imageHandle, resizeCoefficient, ResizedImageHandle2);

				FSDK_GetImageWidth(ResizedImageHandle2, &width2);
				FSDK_GetImageHeight(ResizedImageHandle2, &height2);

				FSDK_FreeImage(imageHandle);

				counterForStaticPhote++;
				if (counterForStaticPhote>1)   
				{
					counterForStaticPhote = 0;
					facePosition2.xc = (int)(resizeCoefficient * facePosition2.xc); 
					facePosition2.yc = (int)(resizeCoefficient * facePosition2.yc);
					facePosition2.w = (int)(resizeCoefficient * facePosition2.w);

					for (int i = 0; i < FSDK_FACIAL_FEATURE_COUNT; i++)
					{
						facialFeatures2[i].x = (int)(resizeCoefficient * facialFeatures2[i].x); 
						facialFeatures2[i].y = (int)(resizeCoefficient * facialFeatures2[i].y);
					}
				}


				ImageOpened2 = true;
				InvalidateRect(hwnd, NULL, TRUE);
			}

			int x1 = facePosition.xc - (int)(facePosition.w*0.6);
			int y1 = facePosition.yc - (int)(facePosition.w*0.5);
			int x2 = facePosition.xc + (int)(facePosition.w*0.6);
			int y2 = facePosition.yc + (int)(facePosition.w*0.7);

			char AttributeValues2[1024];
			int res3 = FSDK_GetTrackerFacialAttribute(tracker, 0, IDs[i], "Gender", AttributeValues2, sizeof(AttributeValues2));

			float ConfidenceMale = 0.0f;
			float ConfidenceFemale = 0.0f;
			FSDK_GetValueConfidence(AttributeValues2, "Male", &ConfidenceMale);
			FSDK_GetValueConfidence(AttributeValues2, "Female", &ConfidenceFemale);

			char str2[1024];
			sprintf_s(str2, sizeof(str2) - 1, "%s, %d%%", (ConfidenceMale > ConfidenceFemale ? "Male" : "Female"),
				ConfidenceMale > ConfidenceFemale ? (int)(ConfidenceMale * 100) : (int)(ConfidenceFemale * 100));

			RECT NameRect3;
			NameRect3.left = x1 - width;
			NameRect3.right = +x2 + width;
			NameRect3.top = 16 + 10 + 25 + _min(height, y2);
			NameRect3.bottom = 16 + 100 + 400 + _min(height, y2 + 30);
			DisplayText2(dc, str2, NameRect3);

			char AttributeValues[1024];
			int res2 = FSDK_GetTrackerFacialAttribute(tracker, 0, IDs[i], "Expression", AttributeValues, sizeof(AttributeValues));


			float ConfidenceSmile = 0.0f;
			float ConfidenceEyesOpen = 0.0f;
			FSDK_GetValueConfidence(AttributeValues, "Smile", &ConfidenceSmile);
			FSDK_GetValueConfidence(AttributeValues, "EyesOpen", &ConfidenceEyesOpen);


			char str[1024];
			sprintf_s(str, sizeof(str) - 1, "Smile: %d%%; Eyes open: %d%%", (int)(ConfidenceSmile * 100), (int)(ConfidenceEyesOpen * 100));

			RECT NameRect2;
			NameRect2.left = x1 - width;
			NameRect2.right = x2 + width;
			NameRect2.top = 16 + 10 + _min(height, y2);
			NameRect2.bottom = 16 + 100 + _min(height, y2 + 30);
			DisplayText2(dc, str, NameRect2);

			FSDK_Features facialFeatures;

			FSDK_GetTrackerFacePosition(tracker, 0, IDs[i], &facePosition);
			FSDK_GetTrackerFacialFeatures(tracker, 0, IDs[i], &facialFeatures);

			SelectObject(dc, FaceRectanglePen);
			SelectObject(dc, FaceRectangleBrush);

			SelectObject(dc, FeatureCirclesPen);

			for (int j = 0; j < FSDK_FACIAL_FEATURE_COUNT; j++)
				Ellipse(dc, facialFeatures[j].x - 2, 16 + facialFeatures[j].y - 2, facialFeatures[j].x + 2, 16 + facialFeatures[j].y + 2);
		}


		int ChoosenUserPathFlag = 0;
		int YonatanPathFlag = 0;
		int DoronPathFlag = 1;

		if (PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			if (msg.message == WM_KEYDOWN && msg.wParam == VK_ESCAPE) 
				break;

			if (msg.message == WM_MOVE)
				InvalidateRect(hwnd, NULL, TRUE);


			if (msg.message == WM_PAINT && ImageOpened2)
			{
				HBITMAP hbitmapHandle; // to store the HBITMAP handle
				FSDK_SaveImageToHBitmap(ResizedImageHandle2, &hbitmapHandle);
				DrawState(dc, NULL, NULL, (LPARAM)hbitmapHandle, NULL, (ClientRect2.right - width2) / 2, 8 + (ClientRect2.bottom - height2) / 2, width2, height2, DST_BITMAP | DSS_NORMAL);
				DeleteObject(hbitmapHandle); // delete the HBITMAP object


				if (FaceDetected2 == FSDKE_OK)
				{
					int left = _min(width2 - 1, max(0, facePosition2.xc - (int)(facePosition2.w*0.6))) + (ClientRect2.right - width2) / 2;
					int right = _min(width2 - 1, max(0, facePosition2.xc + (int)(facePosition2.w*0.6))) + (ClientRect2.right - width2) / 2;
					int top = _min(height2 - 1, max(0, facePosition2.yc - (int)(facePosition2.w*0.5))) + 8 + (ClientRect2.bottom - height2) / 2;
					int bottom = _min(height2 - 1, max(0, facePosition2.yc + (int)(facePosition2.w*0.7))) + 8 + (ClientRect2.bottom - height2) / 2;
					Rectangle(dc, left, top, right, bottom);

					SelectObject(dc, FeatureCirclesPen);
					for (int i = 0; i < FSDK_FACIAL_FEATURE_COUNT; i++)
						Ellipse(dc, _min(width2 - 1, max(0, facialFeatures2[i].x - 2)) + (ClientRect2.right - width2) / 2,
							_min(height2 - 1, max(0, facialFeatures2[i].y - 2)) + 8 + (ClientRect2.bottom - height2) / 2,
							_min(width2 - 1, max(0, facialFeatures2[i].x + 2)) + (ClientRect2.right - width2) / 2,
							_min(height2 - 1, max(0, facialFeatures2[i].y + 2)) + 8 + (ClientRect2.bottom - height2) / 2);
				}
			}

		}


		int RectSelected = GetSelectedRectangle(); 


		if (psRemember == programState && RectSelected >= 0) {
			long long id = IDs[RectSelected];

			if (0 == FSDK_LockID(tracker, id)) {
				strcpy_s(userName, "_noname_");

				DialogBox(NULL, MAKEINTRESOURCE(IDD_DIALOGINPUTNAME), NULL, InputName);
				if (strcmp(userName, "_noname_") != 0) {
					FSDK_SetName(tracker, IDs[RectSelected], userName);
				}
				if (strcmp(userName, "") == 0)
					FSDK_PurgeID(tracker, IDs[RectSelected]);
				FSDK_UnlockID(tracker, IDs[RectSelected]);
			}


		}
		programState = psRecognize;

		FSDK_FreeImage(imageHandle); 

	}

	FSDK_FreeImage(ResizedImageHandle2);

	ReleaseDC(hwnd, dc);
	FSDK_SaveTrackerMemoryToFile(tracker, TrackerMemoryFile);
	FSDK_FreeTracker(tracker);

	if (FSDKE_OK != FSDK_CloseVideoCamera(cameraHandle)) {
		MessageBox(0, L"Error closing camera", L"Error", MB_ICONERROR | MB_OK);
		return -5;
	}

	FSDK_FreeVideoFormatList(VideoFormatList);
	FSDK_FreeCameraList(CameraList, CameraCount);

	FSDK_FinalizeCapturing();
	FSDK_Finalize();

	return 0;
}


LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
	case WM_MOUSEMOVE:
		MouseX = LOWORD(lParam);
		MouseY = HIWORD(lParam);
		return (INT_PTR)TRUE;

	case WM_LBUTTONUP:
		programState = psRemember;
		return (INT_PTR)TRUE;

	case WM_CTLCOLORSTATIC:
		SetBkMode((HDC)wParam, TRANSPARENT);
		return (int)CreateSolidBrush(RGB(230, 230, 230));

	case WM_DESTROY:
		NeedClose = TRUE;
		break;
	}
	return DefWindowProc(hWnd, message, wParam, lParam);
}


INT_PTR CALLBACK InputName(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
	case WM_SHOWWINDOW:
		SetFocus(GetDlgItem(hDlg, IDC_EDIT1));
		return (INT_PTR)TRUE;
		break;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK) {
			int len = SendDlgItemMessageA(hDlg, IDC_EDIT1, WM_GETTEXT, (WPARAM) sizeof(userName) / 2 - 1, (LPARAM)userName);
			userName[len] = '\0';
			EndDialog(hDlg, LOWORD(wParam));
			return (INT_PTR)TRUE;
		}
		if (LOWORD(wParam) == IDCANCEL) {
			EndDialog(hDlg, LOWORD(wParam));
			return (INT_PTR)TRUE;
		}
		break;
	}
	return (INT_PTR)FALSE;
}


